/*!
 * Kantar FocalMeter integration for Snowplow v4.3.1 (http://bit.ly/sp-js)
 * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
 * Licensed under BSD-3-Clause
 */

(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.snowplowFocalMeter = {}));
})(this, (function (exports) { 'use strict';

    /*!
     * Core functionality for Snowplow Browser trackers v4.3.1 (http://bit.ly/sp-js)
     * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
     * Licensed under BSD-3-Clause
     */

    /*
     * Copyright (c) 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
     * All rights reserved.
     *
     * Redistribution and use in source and binary forms, with or without
     * modification, are permitted provided that the following conditions are met:
     *
     * 1. Redistributions of source code must retain the above copyright notice, this
     *    list of conditions and the following disclaimer.
     *
     * 2. Redistributions in binary form must reproduce the above copyright notice,
     *    this list of conditions and the following disclaimer in the documentation
     *    and/or other materials provided with the distribution.
     *
     * 3. Neither the name of the copyright holder nor the names of its
     *    contributors may be used to endorse or promote products derived from
     *    this software without specific prior written permission.
     *
     * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
     * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
     * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
     * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
     * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
     * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
     * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
     * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
     * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
     * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
     */
    /**
     * Attempt to get a value from localStorage
     *
     * @param string - key
     * @returns string The value obtained from localStorage, or
     *                undefined if localStorage is inaccessible
     */
    function attemptGetLocalStorage(key) {
        try {
            var localStorageAlias = window.localStorage, exp = localStorageAlias.getItem(key + '.expires');
            if (exp === null || +exp > Date.now()) {
                return localStorageAlias.getItem(key);
            }
            else {
                localStorageAlias.removeItem(key);
                localStorageAlias.removeItem(key + '.expires');
            }
            return undefined;
        }
        catch (e) {
            return undefined;
        }
    }
    /**
     * Attempt to write a value to localStorage
     *
     * @param string - key
     * @param string - value
     * @param number - ttl Time to live in seconds, defaults to 2 years from Date.now()
     * @returns boolean Whether the operation succeeded
     */
    function attemptWriteLocalStorage(key, value, ttl) {
        if (ttl === void 0) { ttl = 63072000; }
        try {
            var localStorageAlias = window.localStorage, t = Date.now() + ttl * 1000;
            localStorageAlias.setItem("".concat(key, ".expires"), t.toString());
            localStorageAlias.setItem(key, value);
            return true;
        }
        catch (e) {
            return false;
        }
    }
    /*
     * Checks whether localStorage is available, in a way that
     * does not throw a SecurityError in Firefox if "always ask"
     * is enabled for cookies (https://github.com/snowplow/snowplow/issues/163).
     */
    function hasLocalStorage() {
        try {
            return !!window.localStorage;
        }
        catch (e) {
            return true; // SecurityError when referencing it means it exists
        }
    }

    /*
     * Copyright (c) 2023 Snowplow Analytics Ltd, 2010 Anthon Pang
     * All rights reserved.
     *
     * Redistribution and use in source and binary forms, with or without
     * modification, are permitted provided that the following conditions are met:
     *
     * 1. Redistributions of source code must retain the above copyright notice, this
     *    list of conditions and the following disclaimer.
     *
     * 2. Redistributions in binary form must reproduce the above copyright notice,
     *    this list of conditions and the following disclaimer in the documentation
     *    and/or other materials provided with the distribution.
     *
     * 3. Neither the name of the copyright holder nor the names of its
     *    contributors may be used to endorse or promote products derived from
     *    this software without specific prior written permission.
     *
     * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
     * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
     * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
     * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
     * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
     * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
     * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
     * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
     * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
     * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
     */
    var _trackers = {};
    var _configurations = {};
    /**
     * The FocalMeter Plugin
     *
     * The plugin sends requests with the domain user ID to a Kantar endpoint used with the FocalMeter system.
     * A request is made when the first event with a new user ID is tracked.
     *
     * Call `enableFocalMeterIntegration()` to enable the integration with given configuration.
     */
    function FocalMeterPlugin() {
        var LOG;
        var lastUserId;
        var trackerId;
        return {
            activateBrowserPlugin: function (tracker) {
                trackerId = tracker.id;
                _trackers[tracker.id] = tracker;
            },
            logger: function (logger) {
                LOG = logger;
            },
            afterTrack: function (payload) {
                if (!_configurations[trackerId]) {
                    LOG.error('FocalMeter integration not enabled');
                    return;
                }
                var newUserId = payload['duid'];
                var _a = _configurations[trackerId], kantarEndpoint = _a.kantarEndpoint, useLocalStorage = _a.useLocalStorage, processUserId = _a.processUserId;
                if (!lastUserId && useLocalStorage && hasLocalStorage()) {
                    var key = getLocalStorageKey(trackerId);
                    lastUserId = attemptGetLocalStorage(key);
                }
                if (newUserId && newUserId != lastUserId) {
                    lastUserId = newUserId;
                    var processedUserId = processUserId !== undefined ? processUserId(newUserId) : newUserId;
                    sendRequest(kantarEndpoint, processedUserId, LOG, function () {
                        // only write in local storage if the request succeeded
                        if (useLocalStorage && hasLocalStorage()) {
                            var key = getLocalStorageKey(trackerId);
                            attemptWriteLocalStorage(key, newUserId);
                        }
                    });
                }
            },
        };
    }
    /**
     * Enables the integration with Kantar FocalMeter.
     *
     * @param configuration - Configuration with the URL endpoint to send requests to
     * @param trackers - The tracker identifiers which should have the context enabled
     */
    function enableFocalMeterIntegration(configuration, trackers) {
        if (trackers === void 0) { trackers = Object.keys(_trackers); }
        for (var _i = 0, trackers_1 = trackers; _i < trackers_1.length; _i++) {
            var id = trackers_1[_i];
            if (_trackers[id]) {
                _configurations[id] = configuration;
            }
        }
    }
    function getLocalStorageKey(trackerId) {
        return "sp-fclmtr-".concat(trackerId);
    }
    function sendRequest(url, userId, LOG, successCallback) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', getKantarURL(url, userId));
        xhr.timeout = 5000;
        xhr.onreadystatechange = function () {
            if (xhr.readyState === 4 && xhr.status >= 200) {
                if (xhr.status < 300) {
                    successCallback();
                    LOG.debug("ID sent to Kantar: ".concat(userId));
                }
                else {
                    LOG.error("Kantar request failed: ".concat(xhr.status, ": ").concat(xhr.statusText));
                }
            }
        };
        xhr.send();
    }
    function getKantarURL(url, userId) {
        var query = {
            vendor: 'snowplow',
            cs_fpid: userId,
            c12: 'not_set',
        };
        return (url +
            '?' +
            Object.keys(query)
                .map(function (key) { return key + '=' + query[key]; })
                .join('&'));
    }

    exports.FocalMeterPlugin = FocalMeterPlugin;
    exports.enableFocalMeterIntegration = enableFocalMeterIntegration;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=index.umd.js.map
